<?php
/********************************* PHP-P10 ******************************
 *    P10 uplink class by pk910   (c)2011 pk910                         *
 ************************************************************************
 *                          Version 2 (OOP)                             *
 *                                                                      *
 * PHP-P10 is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with PHP-P10; if not, write to the Free Software Foundation,   *
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.       *
 *                                                                      *
 ************************************************************************
 * 
 *  Uplink/Numerics.class.php
 *
 * P10 numeric functions
 *
 ************************************************************************
 * accessable functions
 *
 * static String intToNum(int $int, int $length)
 *     returns the numeric representing $int
 *
 * static int numToInt(String $numeric)
 *     returns the integer value, the numeric represents
 *
 * static String parseIP(String $numeric)
 *     parses an IP Address in numeric format
 *
 * static String numericFromIP(String $ip)
 *     builds a numeric representing the IP
 */

class Numerics {
	private static $base64chars = array(
	  'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P',
	  'Q','R','S','T','U','V','W','X','Y','Z','a','b','c','d','e','f',
	  'g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v',
	  'w','x','y','z','0','1','2','3','4','5','6','7','8','9','[',']'
	);
	private static $base64charsLength = 64;
	
	public static function intToNum($int, $length) {
		//fix a small "bug": normaly 0 = AAAAA but we need 1 = AAAAA
		$int = $int - 1;
		
		$numeric = "";
		for($pos = $length-1; $pos >= 0; $pos--) {
			//current position represents floor($int / ($base64charsLength ^ $pos))
			$base = 1;
			for($i = 0; $i < $pos; $i++) {
				$base = $base * self::$base64charsLength;
			}
			$posValue = floor($int / $base);
			//get the char representing $posValue
			$posChar = self::$base64chars[$posValue];
			$numeric .= $posChar;
		}
		
		return $numeric;
	}
	
	public static function numToInt($numeric) {
		$base = 1;
		$int = 0;
		for($pos = strlen($numeric)-1; $pos >= 0; $pos--) {
			$posValue = array_search($numeric[$pos], self::$base64chars);
			$int = ($posValue * $base);
			$base = $base * self::$base64charsLength;
		}
		
		//fix a small "bug": normaly 0 = AAAAA but we need 1 = AAAAA
		$int = $int + 1;
		
		return $int;
	}
	
	public static function parseIP($numeric, $shortForm = true) {
		if(strlen($numeric) == 6) { //IPv4
			$value = self::numToInt($numeric);
			$ip = array();
			$ip[0] = ($value & 0xff000000) >> 24;
			$ip[1] = ($value & 0x00ff0000) >> 16;
			$ip[2] = ($value & 0x0000ff00) >> 8;
			$ip[3] = ($value & 0x000000ff);
			return implode(".", $ip);
		} else { //IPv6
			$ip = array();
			for($i = 0; $i < strlen($numeric);) {
				if($numeric[$i] == "_") {
					if($shortForm) {
						$ip[] = ""; //::
					} else {
						$rightBlocks = (strlen($numeric) - ($i + 1)) / 3;
						$skipCount = 8 - count($ip) - $rightBlocks;
						for($j = 0; $j < $skipBlocks; $j++) {
							$ip[] = "0";
						}
					}
					$i++;
				} else {
					$value = self::numToInt($numeric[$i].$numeric[$i+1].$numeric[$i+2]);
					$ip[] = dechex($value);
					$i += 3;
				}
			}
			return implode(":", $ip);
		}
	}
	
	public static function numericFromIP($ip) {
		$pattern = array();
		$pattern['ipv6'] = '/^((([0-9A-Fa-f]{1,4}:){7}[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){6}:[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){5}:([0-9A-Fa-f]{1,4}:)?[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){4}:([0-9A-Fa-f]{1,4}:){0,2}[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){3}:([0-9A-Fa-f]{1,4}:){0,3}[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){2}:([0-9A-Fa-f]{1,4}:){0,4}[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){6}((\b((25[0-5])|(1\d{2})|(2[0-4]\d)|(\d{1,2}))\b)\.){3}(\b((25[0-5])|(1\d{2})|(2[0-4]\d)|(\d{1,2}))\b))|(([0-9A-Fa-f]{1,4}:){0,5}:((\b((25[0-5])|(1\d{2})|(2[0-4]\d)|(\d{1,2}))\b)\.){3}(\b((25[0-5])|(1\d{2})|(2[0-4]\d)|(\d{1,2}))\b))|(::([0-9A-Fa-f]{1,4}:){0,5}((\b((25[0-5])|(1\d{2})|(2[0-4]\d)|(\d{1,2}))\b)\.){3}(\b((25[0-5])|(1\d{2})|(2[0-4]\d)|(\d{1,2}))\b))|([0-9A-Fa-f]{1,4}::([0-9A-Fa-f]{1,4}:){0,5}[0-9A-Fa-f]{1,4})|(::([0-9A-Fa-f]{1,4}:){0,6}[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){1,7}:))(|\/[0-9]{1,3})$/';
		$pattern['ipv4'] = '/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}(|\/[0-9]{1,2})$/';
		if(preg_match($pattern['ipv4'], $ip)) {
			//thats quite simple here
			$ip = explode(".", $ip);
			$ipValue = intval($ip[3]);
			$ipValue <<= 8;
			$ipValue |= intval($ip[2]);
			$ipValue <<= 8;
			$ipValue |= intval($ip[1]);
			$ipValue <<= 8;
			$ipValue |= intval($ip[0]);
			$ip = self::intToNum($ipValue,6);
		} else if(preg_match($pattern['ipv6'], $ip)) {
			//thats a little bit complicated :D (we may only have one _)
			$ipv6 = array();
			$ip = explode(":",$ip);
			$last_zero = false; $zero_sequence = 0; $biggest_zero_sequence = 0; $max_start = -1;
			$i = 0;
			foreach($ip as $v) {
				if($v == "") {
					$skipBlocks = (8 - count($ip));
					for($j = 0; $j < $skipBlocks; $j++) {
						$ipv6[$i+$j] = "_";
					}
					$max_start = $i;
					if($last_zero) {
						$zero_sequence += $skipBlocks;
					} else {
						$last_zero = true;
						$zero_sequence = $skipBlocks;
					}
					$i+=$skipBlocks;
					if($zero_sequence > $biggest_zero_sequence) {
						$biggest_zero_sequence = $zero_sequence;
						$max_start = $i-($zero_sequence-1);
					}
				} else {
					$value = hexdec($v);
					if($value == 0) {
						$ipv6[$i] = "_";
						if($last_zero) {
							$zero_sequence++;
						} else {
							$last_zero = true;
							$zero_sequence = 1;
						}
						if($zero_sequence > $biggest_zero_sequence) {
							$biggest_zero_sequence = $zero_sequence;
							$max_start = $i-($zero_sequence-1);
						}
					} else {
						$ipv6[$i] = self::intToNum($value,3);
						$last_zero = false;
					}
					$i++;
				}
			}
			$ip = "";
			for($i = 0; $i < 8; $i++) {
				if($i == $max_start) { //merge the biggest sequence of _'s
					$ip .= "_";
					$i += ($biggest_zero_sequence-1);
				} elseif($ipv6[$i] == "_") {
					$ip .= "AAA";
				} else {
					$ip .= $ipv6[$i];
				}
			}
			return $ip;
		} else {
			$ip = "AAAAAA";
		}
		return $ip;
	}
	
}

?>